package repository

import (
	"app-cloudep-tweeting-service/gen_result/pb/tweeting"
	"app-cloudep-tweeting-service/internal/config"
	"app-cloudep-tweeting-service/internal/domain"
	"app-cloudep-tweeting-service/internal/domain/repository"
	"context"
	"errors"
	"time"

	"github.com/zeromicro/go-zero/core/stores/redis"
)

// TODO 第一版本先使用 Redis 來做，後續如果有效能考量，在考慮使用其他方案

type TimelineRepositoryParam struct {
	Config config.Config
	Redis  redis.Redis
}

type TimelineRepository struct {
	cfg   config.Config
	redis redis.Redis
}

func MustGenerateUseCase(param TimelineRepositoryParam) repository.TimelineRepository {
	return &TimelineRepository{
		cfg:   param.Config,
		redis: param.Redis,
	}
}

// AddPost 將貼文添加到時間線，並根據 Score 排序
func (t *TimelineRepository) AddPost(ctx context.Context, req repository.AddPostRequest) error {
	key := domain.TimelineRedisKey.With(req.UID).ToString()

	// 準備要插入的元素
	zItems := make([]redis.Pair, len(req.PostItems))
	for i, item := range req.PostItems {
		zItems[i] = redis.Pair{
			Score: item.Score,
			Key:   item.PostID,
		}
	}

	// 將 ZSet 元素添加到 Redis
	_, err := t.redis.ZaddsCtx(ctx, key, zItems...)
	if err != nil {
		return err
	}

	// 檢查 ZSet 長度，並在超過 maxLength 時刪除多餘的元素
	if t.cfg.TimelineSetting.MaxLength > 0 {
		// 這裡從 0 到 - (maxLength+1) 代表超過限制的元素範圍
		_, err := t.redis.ZremrangebyrankCtx(ctx, key, 0, -(t.cfg.TimelineSetting.MaxLength + 1))
		if err != nil {
			return err
		}
	}

	// 設置過期時間
	return t.redis.ExpireCtx(ctx, key, int(t.cfg.TimelineSetting.Expire))
}

// FetchTimeline 獲取指定用戶的動態時報
func (t *TimelineRepository) FetchTimeline(ctx context.Context, req repository.FetchTimelineRequest) (repository.FetchTimelineResponse, error) {
	key := domain.TimelineRedisKey.With(req.UID).ToString()

	start := (req.PageIndex - 1) * req.PageSize
	end := start + req.PageSize - 1

	// 從 Redis 中按分數由高到低獲取時間線元素
	pair, err := t.redis.ZrevrangeWithScoresCtx(ctx, key, start, end)
	if err != nil {
		return repository.FetchTimelineResponse{}, err
	}

	// 構建返回結果
	items := make([]repository.TimelineItem, len(pair))
	for i, z := range pair {
		items[i] = repository.TimelineItem{
			PostID: z.Key,
			Score:  z.Score,
		}
	}

	// 計算總數量
	total, err := t.redis.ZcardCtx(ctx, key)
	if err != nil {
		return repository.FetchTimelineResponse{}, err
	}

	return repository.FetchTimelineResponse{
		Items: items,
		Page: tweeting.Pager{
			Total: int64(total),
			Index: req.PageIndex,
			Size:  req.PageSize,
		},
	}, nil
}

// SetNoMoreDataFlag 標記時間線已完整，避免繼續查詢資料庫
func (t *TimelineRepository) SetNoMoreDataFlag(ctx context.Context, uid string) error {
	key := domain.TimelineRedisKey.With(uid).ToString()

	// 添加一個標誌到時間線的 ZSet
	_, err := t.redis.ZaddsCtx(ctx, key, redis.Pair{
		Score: time.Now().UTC().Unix(),
		Key:   domain.LastOfTimelineFlag,
	})
	if err != nil {
		return err
	}

	// 設置過期時間
	return t.redis.ExpireCtx(ctx, key, int(t.cfg.TimelineSetting.Expire))
}

// HasNoMoreData 檢查時間線是否已完整，決定是否需要查詢資料庫
func (t *TimelineRepository) HasNoMoreData(ctx context.Context, uid string) (bool, error) {
	key := domain.TimelineRedisKey.With(uid).ToString()

	// 檢查 "NoMoreData" 標誌是否存在
	score, err := t.redis.ZscoreCtx(ctx, key, domain.LastOfTimelineFlag)
	if errors.Is(err, redis.Nil) {
		return false, nil // 標誌不存在
	}
	if err != nil {
		return false, err // 其他錯誤
	}

	return score != 0, nil
}

// ClearNoMoreDataFlag 清除時間線的 "NoMoreData" 標誌
func (t *TimelineRepository) ClearNoMoreDataFlag(ctx context.Context, uid string) error {
	key := domain.TimelineRedisKey.With(uid).ToString()
	// 移除 "NoMoreData" 標誌
	_, err := t.redis.ZremCtx(ctx, key, domain.LastOfTimelineFlag)
	return err
}
