package repository

import (
	"app-cloudep-tweeting-service/internal/config"
	"app-cloudep-tweeting-service/internal/domain"
	"app-cloudep-tweeting-service/internal/domain/repository"
	"context"
	"github.com/alicebob/miniredis/v2"
	"github.com/stretchr/testify/assert"
	"github.com/zeromicro/go-zero/core/stores/redis"
	"testing"
	"time"
)

func NewRepo() (*miniredis.Miniredis, repository.TimelineRepository, error) {
	r1, err := miniredis.Run()
	if err != nil {
		return nil, nil, err
	}

	newRedis, err := redis.NewRedis(redis.RedisConf{
		Host: r1.Addr(),
		Type: redis.ClusterType,
		Pass: "",
	})
	if err != nil {
		r1.Close()
		return nil, nil, err
	}

	c := config.Config{
		TimelineSetting: struct {
			Expire    int64
			MaxLength int64
		}{Expire: 86400, MaxLength: 1000},
	}

	timelineRepo := MustGenerateUseCase(TimelineRepositoryParam{
		Config: c,
		Redis:  *newRedis,
	})

	return r1, timelineRepo, nil
}

func TestAddPost(t *testing.T) {
	tests := []struct {
		name      string
		action    func(t *testing.T, repo repository.TimelineRepository) error
		expectErr bool
		validate  func(t *testing.T, r1 *miniredis.Miniredis)
	}{
		{
			name: "success",
			action: func(t *testing.T, repo repository.TimelineRepository) error {
				ctx := context.Background()
				uid := "OOOOOOKJ"

				return repo.AddPost(ctx, repository.AddPostRequest{
					UID: uid,
					PostItems: []repository.TimelineItem{
						{PostID: "post1", Score: 100},
					},
				})
			},
			expectErr: false,
			validate: func(t *testing.T, r1 *miniredis.Miniredis) {
				uid := "OOOOOOKJ"
				key := domain.TimelineRedisKey.With(uid).ToString()
				score, err := r1.ZScore(key, "post1")
				assert.NoError(t, err)
				assert.Equal(t, float64(100), score)
			},
		},
		{
			name: "timeout",
			action: func(t *testing.T, repo repository.TimelineRepository) error {
				ctx := context.Background()
				timeoutCtx, cancel := context.WithTimeout(ctx, 1*time.Millisecond)
				defer cancel()
				time.Sleep(2 * time.Millisecond)

				uid := "OOOOOLK"
				return repo.AddPost(timeoutCtx, repository.AddPostRequest{
					UID: uid,
					PostItems: []repository.TimelineItem{
						{PostID: "post2", Score: 200},
					},
				})
			},
			expectErr: true,
		},
		{
			name: "Redis error on Zadd",
			action: func(t *testing.T, repo repository.TimelineRepository) error {
				r1, repo, err := NewRepo()
				assert.NoError(t, err)
				r1.Close() // 模拟 Redis 错误

				ctx := context.Background()
				uid := "OOOOOWE"
				return repo.AddPost(ctx, repository.AddPostRequest{
					UID: uid,
					PostItems: []repository.TimelineItem{
						{PostID: "post3", Score: 300},
					},
				})
			},
			expectErr: true,
		},
	}

	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			r1, repo, err := NewRepo()
			assert.NoError(t, err)
			defer r1.Close()

			err = tt.action(t, repo)

			if tt.expectErr {
				assert.Error(t, err)
			} else {
				assert.NoError(t, err)
				if tt.validate != nil {
					tt.validate(t, r1)
				}
			}
		})
	}
}

func TestFetchTimeline(t *testing.T) {
	tests := []struct {
		name      string
		action    func(t *testing.T, repo repository.TimelineRepository) (repository.FetchTimelineResponse, error)
		expectErr bool
		validate  func(t *testing.T, r1 *miniredis.Miniredis, resp *repository.FetchTimelineResponse)
	}{
		{
			name: "FetchTimeline - success",
			action: func(t *testing.T, repo repository.TimelineRepository) (repository.FetchTimelineResponse, error) {
				ctx := context.Background()
				uid := "user123"

				_ = repo.AddPost(ctx, repository.AddPostRequest{
					UID: uid,
					PostItems: []repository.TimelineItem{
						{PostID: "post1", Score: 200},
						{PostID: "post2", Score: 100},
					},
				})

				return repo.FetchTimeline(ctx, repository.FetchTimelineRequest{
					UID:       uid,
					PageSize:  10,
					PageIndex: 1,
				})
			},
			expectErr: false,
			validate: func(t *testing.T, r1 *miniredis.Miniredis, resp *repository.FetchTimelineResponse) {
				assert.Equal(t, 2, len(resp.Items))
				assert.Equal(t, "post1", resp.Items[0].PostID)
				assert.Equal(t, "post2", resp.Items[1].PostID)
			},
		},
		{
			name: "FetchTimeline - timeout",
			action: func(t *testing.T, repo repository.TimelineRepository) (repository.FetchTimelineResponse, error) {
				ctx := context.Background()
				timeoutCtx, cancel := context.WithTimeout(ctx, 1*time.Millisecond)
				defer cancel()
				time.Sleep(2 * time.Millisecond)

				uid := "user123"
				return repo.FetchTimeline(timeoutCtx, repository.FetchTimelineRequest{
					UID:       uid,
					PageSize:  10,
					PageIndex: 1,
				})
			},
			expectErr: true,
		},
		{
			name: "FetchTimeline - Redis error on ZrangebyscoreWithScoresCtx",
			action: func(t *testing.T, repo repository.TimelineRepository) (repository.FetchTimelineResponse, error) {
				r1, repo, err := NewRepo()
				assert.NoError(t, err)
				defer r1.Close()

				uid := "user123"

				_ = repo.AddPost(context.Background(), repository.AddPostRequest{
					UID: uid,
					PostItems: []repository.TimelineItem{
						{PostID: "post3", Score: 300},
					},
				})

				r1.Close()

				return repo.FetchTimeline(context.Background(), repository.FetchTimelineRequest{
					UID:       uid,
					PageSize:  10,
					PageIndex: 1,
				})
			},
			expectErr: true,
		},
		{
			name: "FetchTimeline - Redis error on ZcardCtx",
			action: func(t *testing.T, repo repository.TimelineRepository) (repository.FetchTimelineResponse, error) {
				r1, repo, err := NewRepo()
				assert.NoError(t, err)
				defer r1.Close()

				uid := "user123"

				_ = repo.AddPost(context.Background(), repository.AddPostRequest{
					UID: uid,
					PostItems: []repository.TimelineItem{
						{PostID: "post4", Score: 400},
					},
				})

				r1.Close()

				return repo.FetchTimeline(context.Background(), repository.FetchTimelineRequest{
					UID:       uid,
					PageSize:  10,
					PageIndex: 1,
				})
			},
			expectErr: true,
		},
	}

	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			r1, repo, err := NewRepo()
			assert.NoError(t, err)
			defer r1.Close()

			resp, err := tt.action(t, repo)

			if tt.expectErr {
				assert.Error(t, err)
			} else {
				assert.NoError(t, err)
				if tt.validate != nil {
					tt.validate(t, r1, &resp)
				}
			}
		})
	}
}

func TestSetNoMoreDataFlag(t *testing.T) {
	tests := []struct {
		name      string
		action    func(t *testing.T, repo repository.TimelineRepository) error
		expectErr bool
		validate  func(t *testing.T, r1 *miniredis.Miniredis)
	}{
		{
			name: "SetNoMoreDataFlag - success",
			action: func(t *testing.T, repo repository.TimelineRepository) error {
				ctx := context.Background()
				uid := "user123"
				return repo.SetNoMoreDataFlag(ctx, uid)
			},
			expectErr: false,
			validate: func(t *testing.T, r1 *miniredis.Miniredis) {
				uid := "user123"
				key := domain.TimelineRedisKey.With(uid).ToString()
				score, _ := r1.ZScore(key, domain.LastOfTimelineFlag)
				assert.NotZero(t, score)

				// 驗證是否設定過期時間
				ttl := r1.TTL(key)
				assert.Equal(t, time.Duration(86400)*time.Second, ttl)
			},
		},
		{
			name: "SetNoMoreDataFlag - Redis error on ZaddsCtx",
			action: func(t *testing.T, repo repository.TimelineRepository) error {
				r1, repo, err := NewRepo()
				assert.NoError(t, err)
				r1.Close() // 手動關閉，復現錯誤

				ctx := context.Background()
				uid := "user123"
				return repo.SetNoMoreDataFlag(ctx, uid)
			},
			expectErr: true,
		},
		{
			name: "SetNoMoreDataFlag - Redis error on ExpireCtx",
			action: func(t *testing.T, repo repository.TimelineRepository) error {
				r1, repo, err := NewRepo()
				assert.NoError(t, err)

				ctx := context.Background()
				uid := "user123"

				r1.Close()

				return repo.SetNoMoreDataFlag(ctx, uid)
			},
			expectErr: true,
		},
	}

	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			r1, repo, err := NewRepo()
			assert.NoError(t, err)
			defer r1.Close()

			err = tt.action(t, repo)

			if tt.expectErr {
				assert.Error(t, err)
			} else {
				assert.NoError(t, err)
				if tt.validate != nil {
					tt.validate(t, r1)
				}
			}
		})
	}
}

func TestHasNoMoreData(t *testing.T) {
	tests := []struct {
		name      string
		action    func(t *testing.T, repo repository.TimelineRepository) (bool, error)
		expectErr bool
		expected  bool
		setup     func(r1 *miniredis.Miniredis)
	}{
		{
			name: "HasNoMoreData - 標誌存在",
			action: func(t *testing.T, repo repository.TimelineRepository) (bool, error) {
				ctx := context.Background()
				uid := "user123"
				return repo.HasNoMoreData(ctx, uid)
			},
			expectErr: false,
			expected:  true,
			setup: func(r1 *miniredis.Miniredis) {
				uid := "user123"
				key := domain.TimelineRedisKey.With(uid).ToString()
				_, _ = r1.ZAdd(key, float64(time.Now().UTC().Unix()), domain.LastOfTimelineFlag)
			},
		},
		{
			name: "HasNoMoreData - 標誌不存在",
			action: func(t *testing.T, repo repository.TimelineRepository) (bool, error) {
				ctx := context.Background()
				uid := "user123"
				return repo.HasNoMoreData(ctx, uid)
			},
			expectErr: false,
			expected:  false,
			setup:     func(r1 *miniredis.Miniredis) {}, // 不設置標誌
		},
	}

	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			r1, repo, err := NewRepo()
			assert.NoError(t, err)
			defer r1.Close()

			tt.setup(r1)

			result, err := tt.action(t, repo)

			if tt.expectErr {
				assert.Error(t, err)
			} else {
				assert.NoError(t, err)
				assert.Equal(t, tt.expected, result)
			}
		})
	}
}

func TestClearNoMoreDataFlag(t *testing.T) {
	tests := []struct {
		name      string
		action    func(t *testing.T, repo repository.TimelineRepository) error
		expectErr bool
		setup     func(r1 *miniredis.Miniredis)
		validate  func(t *testing.T, r1 *miniredis.Miniredis)
	}{
		{
			name: "ClearNoMoreDataFlag - 成功清除標誌",
			action: func(t *testing.T, repo repository.TimelineRepository) error {
				ctx := context.Background()
				uid := "user123"
				return repo.ClearNoMoreDataFlag(ctx, uid)
			},
			expectErr: false,
			setup: func(r1 *miniredis.Miniredis) {
				uid := "user123"
				key := domain.TimelineRedisKey.With(uid).ToString()
				_, err := r1.ZAdd(key, 100, domain.LastOfTimelineFlag) // 設置標誌
				assert.NoError(t, err)
			},
			validate: func(t *testing.T, r1 *miniredis.Miniredis) {
				uid := "user123"
				key := domain.TimelineRedisKey.With(uid).ToString()
				_, err := r1.ZScore(key, domain.LastOfTimelineFlag)
				assert.Error(t, err) // 標誌應該已被移除
			},
		},
	}

	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			r1, repo, err := NewRepo()
			assert.NoError(t, err)
			defer r1.Close()

			tt.setup(r1)

			err = tt.action(t, repo)

			if tt.expectErr {
				assert.Error(t, err)
			} else {
				assert.NoError(t, err)
				tt.validate(t, r1)
			}
		})
	}
}
